@@ -44,9 +44,7 @@ disk_size=$(openstack flavor show baremetal -f value -c disk)
 ephemeral_size=$(openstack flavor show baremetal -f value -c "OS-FLV-EXT-DATA:ephemeral")
 expected_local_gb=$(($disk_size + $ephemeral_size))
 
-# FIXME(dtantsur): switch to OSC as soon as `openstack endpoint list` actually
-# works on devstack
-ironic_url=$(keystone endpoint-get --service baremetal | tail -n +4 | head -n -1 | tr '|' ' ' | awk '{ print $2; }')
+ironic_url=$(openstack endpoint show baremetal -f value -c publicurl)
 if [ -z "$ironic_url" ]; then
     echo "Cannot find Ironic URL"
     exit 1
@@ -117,7 +115,7 @@ openstack baremetal introspection rule purge
 
 function test_swift {
     # Basic sanity check of the data stored in Swift
-    stored_data_json=$(curl_ins GET v1/introspection/$uuid/data)
+    stored_data_json=$(openstack baremetal introspection data save $uuid)
     stored_cpu_arch=$(echo $stored_data_json | jq -r '.cpu_arch')
     echo CPU arch for $uuid from stored data: $stored_cpu_arch
     if [ "$stored_cpu_arch" != "$expected_cpu_arch" ]; then
