@@ -30,7 +30,9 @@ IRONIC_INSPECTOR_INTERNAL_SUBNET_SIZE=${IRONIC_INSPECTOR_INTERNAL_SUBNET_SIZE:-2
 IRONIC_INSPECTOR_DHCP_RANGE=${IRONIC_INSPECTOR_DHCP_RANGE:-172.24.42.100,172.24.42.253}
 IRONIC_INSPECTOR_INTERFACE=${IRONIC_INSPECTOR_INTERFACE:-br-inspector}
 IRONIC_INSPECTOR_INTERNAL_URI="http://$IRONIC_INSPECTOR_INTERNAL_IP:$IRONIC_INSPECTOR_PORT"
-IRONIC_INSPECTOR_INTERNAL_IP_WITH_NET=$IRONIC_INSPECTOR_INTERNAL_IP/$IRONIC_INSPECTOR_INTERNAL_SUBNET_SIZE
+IRONIC_INSPECTOR_INTERNAL_IP_WITH_NET="$IRONIC_INSPECTOR_INTERNAL_IP/$IRONIC_INSPECTOR_INTERNAL_SUBNET_SIZE"
+
+IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK=${IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK:-""}
 
 GITDIR["python-ironic-inspector-client"]=$DEST/python-ironic-inspector-client
 GITREPO["python-ironic-inspector-client"]=${IRONIC_INSPECTOR_CLIENT_REPO:-${GIT_BASE}/openstack/python-ironic-inspector-client.git}
@@ -134,11 +136,25 @@ function prepare_tftp {
         fi
     fi
 
-    mkdir_chown_stack "$IRONIC_TFTPBOOT_DIR/pxelinux.cfg"
-    cp $IRONIC_INSPECTOR_KERNEL_PATH $IRONIC_TFTPBOOT_DIR/ironic-inspector.kernel
-    cp $IRONIC_INSPECTOR_INITRAMFS_PATH $IRONIC_TFTPBOOT_DIR
+    if [[ "$IRONIC_IPXE_ENABLED" == "True" ]] ; then
+        cp $IRONIC_INSPECTOR_KERNEL_PATH $IRONIC_HTTP_DIR/ironic-inspector.kernel
+        cp $IRONIC_INSPECTOR_INITRAMFS_PATH $IRONIC_HTTP_DIR
 
-    cat > "$IRONIC_TFTPBOOT_DIR/pxelinux.cfg/default" <<EOF
+        cat > "$IRONIC_HTTP_DIR/ironic-inspector.ipxe" <<EOF
+#!ipxe
+
+dhcp
+
+kernel http://$IRONIC_HTTP_SERVER:$IRONIC_HTTP_PORT/ironic-inspector.kernel BOOTIF=\${mac} $IRONIC_INSPECTOR_KERNEL_CMDLINE
+initrd http://$IRONIC_HTTP_SERVER:$IRONIC_HTTP_PORT/ironic-inspector.initramfs
+boot
+EOF
+    else
+        mkdir_chown_stack "$IRONIC_TFTPBOOT_DIR/pxelinux.cfg"
+        cp $IRONIC_INSPECTOR_KERNEL_PATH $IRONIC_TFTPBOOT_DIR/ironic-inspector.kernel
+        cp $IRONIC_INSPECTOR_INITRAMFS_PATH $IRONIC_TFTPBOOT_DIR
+
+        cat > "$IRONIC_TFTPBOOT_DIR/pxelinux.cfg/default" <<EOF
 default inspect
 
 label inspect
@@ -147,6 +163,19 @@ append initrd=ironic-inspector.initramfs $IRONIC_INSPECTOR_KERNEL_CMDLINE
 
 ipappend 3
 EOF
+    fi
+}
+
+function inspector_configure_auth_for {
+    inspector_iniset $1 auth_type password
+    inspector_iniset $1 auth_url "$KEYSTONE_SERVICE_URI"
+    inspector_iniset $1 username $IRONIC_INSPECTOR_ADMIN_USER
+    inspector_iniset $1 password $SERVICE_PASSWORD
+    inspector_iniset $1 project_name $SERVICE_PROJECT_NAME
+    inspector_iniset $1 user_domain_id default
+    inspector_iniset $1 project_domain_id default
+    inspector_iniset $1 cafile $SSL_BUNDLE_FILE
+    inspector_iniset $1 os_region $REGION_NAME
 }
 
 function configure_inspector {
@@ -157,11 +186,7 @@ function configure_inspector {
 
     cp "$IRONIC_INSPECTOR_DIR/example.conf" "$IRONIC_INSPECTOR_CONF_FILE"
     inspector_iniset DEFAULT debug $IRONIC_INSPECTOR_DEBUG
-    inspector_iniset ironic os_auth_url "$KEYSTONE_SERVICE_URI"
-    inspector_iniset ironic os_username $IRONIC_INSPECTOR_ADMIN_USER
-    inspector_iniset ironic os_password $SERVICE_PASSWORD
-    inspector_iniset ironic os_tenant_name $SERVICE_PROJECT_NAME
-
+    inspector_configure_auth_for ironic
     configure_auth_token_middleware $IRONIC_INSPECTOR_CONF_FILE $IRONIC_INSPECTOR_ADMIN_USER $IRONIC_INSPECTOR_AUTH_CACHE_DIR/api
 
     inspector_iniset DEFAULT listen_port $IRONIC_INSPECTOR_PORT
@@ -198,26 +223,39 @@ function configure_inspector {
     inspector_iniset processing ramdisk_logs_dir "$IRONIC_INSPECTOR_RAMDISK_LOGDIR"
     inspector_iniset processing always_store_ramdisk_logs "$IRONIC_INSPECTOR_ALWAYS_STORE_RAMDISK_LOGS"
     inspector_iniset processing log_bmc_address False
+    if [ -n "$IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK" ]; then
+        inspector_iniset processing node_not_found_hook "$IRONIC_INSPECTOR_NODE_NOT_FOUND_HOOK"
+    fi
     inspector_iniset DEFAULT timeout $IRONIC_INSPECTOR_TIMEOUT
 
     get_or_create_service "ironic-inspector" "baremetal-introspection" "Ironic Inspector baremetal introspection service"
     get_or_create_endpoint "baremetal-introspection" "$REGION_NAME" \
         "$IRONIC_INSPECTOR_URI" "$IRONIC_INSPECTOR_URI" "$IRONIC_INSPECTOR_URI"
+
 }
 
 function configure_inspector_swift {
-    inspector_iniset swift os_auth_url "$KEYSTONE_SERVICE_URI/v2.0"
-    inspector_iniset swift username $IRONIC_INSPECTOR_ADMIN_USER
-    inspector_iniset swift password $SERVICE_PASSWORD
-    inspector_iniset swift tenant_name $SERVICE_PROJECT_NAME
-
+    inspector_configure_auth_for swift
     inspector_iniset processing store_data swift
 }
 
 function configure_inspector_dhcp {
     mkdir_chown_stack "$IRONIC_INSPECTOR_CONF_DIR"
 
-    cat > "$IRONIC_INSPECTOR_DHCP_CONF_FILE" <<EOF
+    if [[ "$IRONIC_IPXE_ENABLED" == "True" ]] ; then
+        cat > "$IRONIC_INSPECTOR_DHCP_CONF_FILE" <<EOF
+no-daemon
+port=0
+interface=$IRONIC_INSPECTOR_INTERFACE
+bind-interfaces
+dhcp-range=$IRONIC_INSPECTOR_DHCP_RANGE
+dhcp-match=ipxe,175
+dhcp-boot=tag:!ipxe,undionly.kpxe
+dhcp-boot=tag:ipxe,http://$IRONIC_HTTP_SERVER:$IRONIC_HTTP_PORT/ironic-inspector.ipxe
+dhcp-sequential-ip
+EOF
+    else
+        cat > "$IRONIC_INSPECTOR_DHCP_CONF_FILE" <<EOF
 no-daemon
 port=0
 interface=$IRONIC_INSPECTOR_INTERFACE
@@ -226,6 +264,7 @@ dhcp-range=$IRONIC_INSPECTOR_DHCP_RANGE
 dhcp-boot=pxelinux.0
 dhcp-sequential-ip
 EOF
+    fi
 }
 
 function prepare_environment {
@@ -256,6 +295,9 @@ function create_ironic_inspector_cache_dir {
 function cleanup_inspector {
     rm -f $IRONIC_TFTPBOOT_DIR/pxelinux.cfg/default
     rm -f $IRONIC_TFTPBOOT_DIR/ironic-inspector.*
+    if [[ "$IRONIC_IPXE_ENABLED" == "True" ]] ; then
+        rm -f $IRONIC_HTTP_DIR/ironic-inspector.*
+    fi
     sudo rm -f /etc/sudoers.d/ironic-inspector-rootwrap
     sudo rm -rf $IRONIC_INSPECTOR_AUTH_CACHE_DIR
     sudo rm -rf "$IRONIC_INSPECTOR_RAMDISK_LOGDIR"
