@@ -13,9 +13,6 @@
 
 """Standard set of plugins."""
 
-import base64
-import datetime
-import os
 import sys
 
 import netaddr
@@ -56,15 +53,6 @@ class RootDiskSelectionHook(base.ProcessingHook):
                   'as an inspection ramdisk'),
                 node_info=node_info, data=introspection_data)
 
-        if 'size' in hints:
-            # Special case to match IPA behaviour
-            try:
-                hints['size'] = int(hints['size'])
-            except (TypeError, ValueError):
-                raise utils.Error(_('Invalid root device size hint, expected '
-                                    'an integer, got %s') % hints['size'],
-                                  node_info=node_info, data=introspection_data)
-
         disks = inventory.get('disks', [])
         if not disks:
             raise utils.Error(_('No disks found'),
@@ -200,6 +188,20 @@ class ValidateInterfacesHook(base.ProcessingHook):
                               iface, data=data)
                     continue
 
+                if not mac:
+                    LOG.debug('Skipping interface %s without link information',
+                              name, data=data)
+                    continue
+
+                if not utils.is_valid_mac(mac):
+                    LOG.warning(_LW('MAC %(mac)s for interface %(name)s is '
+                                    'not valid, skipping'),
+                                {'mac': mac, 'name': name},
+                                data=data)
+                    continue
+
+                mac = mac.lower()
+
                 LOG.debug('Found interface %(name)s with MAC "%(mac)s" and '
                           'IP address "%(ip)s"',
                           {'name': name, 'mac': mac, 'ip': ip}, data=data)
@@ -232,20 +234,6 @@ class ValidateInterfacesHook(base.ProcessingHook):
             mac = iface.get('mac')
             ip = iface.get('ip')
 
-            if not mac:
-                LOG.debug('Skipping interface %s without link information',
-                          name, data=data)
-                continue
-
-            if not utils.is_valid_mac(mac):
-                LOG.warning(_LW('MAC %(mac)s for interface %(name)s is not '
-                                'valid, skipping'),
-                            {'mac': mac, 'name': name},
-                            data=data)
-                continue
-
-            mac = mac.lower()
-
             if name == 'lo' or (ip and netaddr.IPAddress(ip).is_loopback()):
                 LOG.debug('Skipping local interface %s', name, data=data)
                 continue
@@ -320,40 +308,8 @@ class ValidateInterfacesHook(base.ProcessingHook):
 class RamdiskErrorHook(base.ProcessingHook):
     """Hook to process error send from the ramdisk."""
 
-    DATETIME_FORMAT = '%Y.%m.%d_%H.%M.%S_%f'
-
     def before_processing(self, introspection_data, **kwargs):
         error = introspection_data.get('error')
-        logs = introspection_data.get('logs')
-
-        if error or CONF.processing.always_store_ramdisk_logs:
-            if logs:
-                self._store_logs(logs, introspection_data)
-            else:
-                LOG.debug('No logs received from the ramdisk',
-                          data=introspection_data)
-
         if error:
             raise utils.Error(_('Ramdisk reported error: %s') % error,
                               data=introspection_data)
-
-    def _store_logs(self, logs, introspection_data):
-        if not CONF.processing.ramdisk_logs_dir:
-            LOG.warning(
-                _LW('Failed to store logs received from the ramdisk '
-                    'because ramdisk_logs_dir configuration option '
-                    'is not set'),
-                data=introspection_data)
-            return
-
-        if not os.path.exists(CONF.processing.ramdisk_logs_dir):
-            os.makedirs(CONF.processing.ramdisk_logs_dir)
-
-        time_fmt = datetime.datetime.utcnow().strftime(self.DATETIME_FORMAT)
-        bmc_address = introspection_data.get('ipmi_address', 'unknown')
-        file_name = 'bmc_%s_%s' % (bmc_address, time_fmt)
-        with open(os.path.join(CONF.processing.ramdisk_logs_dir, file_name),
-                  'wb') as fp:
-            fp.write(base64.b64decode(logs))
-        LOG.info(_LI('Ramdisk logs stored in file %s'), file_name,
-                 data=introspection_data)
