@@ -18,6 +18,7 @@ import futurist
 from keystonemiddleware import auth_token
 from oslo_config import cfg
 from oslo_log import log
+from oslo_middleware import cors as cors_middleware
 import six
 
 from ironic_inspector.common.i18n import _, _LE
@@ -159,6 +160,17 @@ def add_auth_middleware(app):
     app.wsgi_app = auth_token.AuthProtocol(app.wsgi_app, auth_conf)
 
 
+def add_cors_middleware(app):
+    """Create a CORS wrapper
+
+    Attach ironic-inspector-specific defaults that must be included
+    in all CORS responses.
+
+    :param app: application
+    """
+    app.wsgi_app = cors_middleware.CORS(app.wsgi_app, CONF)
+
+
 def check_auth(request):
     """Check authentication on request.
 
@@ -186,3 +198,10 @@ def get_auth_strategy():
     if CONF.authenticate is not None:
         return 'keystone' if CONF.authenticate else 'noauth'
     return CONF.auth_strategy
+
+
+def get_valid_macs(data):
+    """Get a list of valid MAC's from the introspection data."""
+    return [m['mac']
+            for m in data.get('all_interfaces', {}).values()
+            if m.get('mac')]
